@@ -32,6 +32,7 @@
 
 BasisFunc::BasisFunc()
   :
+  Nd_(0),
   Ny_(0),
   kx_(0),
   kz_(0),
@@ -41,13 +42,12 @@ BasisFunc::BasisFunc()
   b_(0),
   state_(Spectral),
   lambda_(0),
-  u_(),
-  v_(),
-  w_()
+  u_(0)
 {}
 
 BasisFunc::BasisFunc(int Ny, const BasisFunc& f)
   :
+  Nd_(f.Nd_),
   Ny_(Ny),
   kx_(f.kx_),
   kz_(f.kz_),
@@ -57,14 +57,13 @@ BasisFunc::BasisFunc(int Ny, const BasisFunc& f)
   b_(f.b_),
   state_(f.state_),
   lambda_(f.lambda_),
-  u_(Ny, f.u_),
-  v_(Ny, f.v_),
-  w_(Ny, f.w_)
+  u_(f.Nd())
 {}
 
-BasisFunc::BasisFunc(int Ny, int kx, int kz, Real Lx, Real Lz, Real a, Real b, 
-	    fieldstate s)
+BasisFunc::BasisFunc(int Nd, int Ny, int kx, int kz, Real Lx, Real Lz, 
+		     Real a, Real b, fieldstate s)
   :
+  Nd_(Nd),
   Ny_(Ny),
   kx_(kx),
   kz_(kz),
@@ -74,15 +73,34 @@ BasisFunc::BasisFunc(int Ny, int kx, int kz, Real Lx, Real Lz, Real a, Real b,
   b_(b),
   state_(s),
   lambda_(0.0),
-  u_(Ny, a, b, s),
-  v_(Ny, a, b, s),
-  w_(Ny, a, b, s)
-{}
-
+  u_(Nd)
+{
+  for (int n=0; n<Nd_; ++n) 
+    u_[n] = ComplexChebyCoeff(Ny_,a_,b_,state_);
+}
 
+BasisFunc::BasisFunc(int Ny, int kx, int kz, Real Lx, Real Lz, 
+		     Real a, Real b, fieldstate s)
+  :
+  Nd_(3),
+  Ny_(Ny),
+  kx_(kx),
+  kz_(kz),
+  Lx_(Lx),
+  Lz_(Lz),
+  a_(a),
+  b_(b),
+  state_(s),
+  lambda_(0.0),
+  u_(3)
+{
+  for (int n=0; n<Nd_; ++n) 
+    u_[n] = ComplexChebyCoeff(Ny_,a_,b_,state_);
+}
 
 BasisFunc::BasisFunc(const string& filebase) 
 :
+  Nd_(0),
   Ny_(0),
   kx_(0),
   kz_(0),
@@ -92,9 +110,7 @@ BasisFunc::BasisFunc(const string& filebase)
   b_(0),
   state_(Spectral),
   lambda_(0),
-  u_(),
-  v_(),
-  w_()
+  u_(0)
 {
   string filename = filebase + string(".bf");
   ifstream is(filename.c_str());
@@ -108,8 +124,8 @@ BasisFunc::BasisFunc(const string& filebase)
     cerr << "BasisFunc::BasisFunc(filebase): bad header in file " << filename << endl;
     assert(false);
   }
-  int newNy;
-  is >> newNy
+  is >> Nd_
+     >> Ny_
      >> kx_
      >> kz_
      >> Lx_
@@ -118,21 +134,20 @@ BasisFunc::BasisFunc(const string& filebase)
      >> b_
      >> state_ 
      >> lambda_;
-  u_.setBounds(a_,b_);
-  v_.setBounds(a_,b_);
-  w_.setBounds(a_,b_);
-  u_.setState(state_);
-  v_.setState(state_);
-  w_.setState(state_);
+  u_.resize(Nd_);
+  for (int n=0; n<Nd_; ++n) {
+    u_[n].resize(Ny_);
+    u_[n].setBounds(a_,b_);
+    u_[n].setState(state_);
+  }
 
-  resize(newNy);
   Real r;
   Real i;
-  for (int n=0; n<Ny_; ++n) {
-    is >> r >> i; u_.set(n,r+I*i);
-    is >> r >> i; v_.set(n,r+I*i);
-    is >> r >> i; w_.set(n,r+I*i);
-  }
+  for (int ny=0; ny<Ny_; ++ny) 
+    for (int n=0; n<Nd_; ++n) {
+      is >> r >> i; 
+      u_[n].set(ny,Complex(r,i));
+    }
 }
 
 
@@ -162,6 +177,7 @@ BasisFunc::BasisFunc(const ComplexChebyCoeff& u,
 		     const ComplexChebyCoeff& w,
 		     Real lam, int kx, int kz, Real Lx, Real Lz) 
 :
+  Nd_(3),
   Ny_(u.numModes()),
   kx_(kx),
   kz_(kz),
@@ -171,11 +187,15 @@ BasisFunc::BasisFunc(const ComplexChebyCoeff& u,
   b_(u.b()),
   state_(u.state()),
   lambda_(lam),
-  u_(u),
-  v_(v),
-  w_(w)
+  u_(3)
 {
-  assert(u.congruent(v) && u.congruent(w));
+  u_[0] = u;
+  u_[1] = v;
+  u_[2] = w;
+  for (int n=1; n<Nd_; ++n) {
+    assert(u_[0].congruent(u_[n]));
+    ;  // this statement needed for optimized compilation, #define assert(x) ;
+  }
 }
 
 void BasisFunc::save(const string& filebase, fieldstate savestate) const {
@@ -188,6 +208,7 @@ void BasisFunc::save(const string& filebase, fieldstate savestate) const {
   os << scientific << setprecision(REAL_DIGITS);
   char sp=' ';
   os << '%'
+     <<sp<< Nd_
      <<sp<< Ny_
      <<sp<< kx_
      <<sp<< kz_
@@ -199,29 +220,18 @@ void BasisFunc::save(const string& filebase, fieldstate savestate) const {
      <<sp<< lambda_
      << '\n';
 
-  for (int i=0; i<Ny_; ++i) 
-    os << Re(u_[i]) <<sp<< Im(u_[i]) <<sp
-       << Re(v_[i]) <<sp<< Im(v_[i]) <<sp
-       << Re(w_[i]) <<sp<< Im(w_[i]) <<'\n';
+  for (int ny=0; ny<Ny_; ++ny) {
+    for (int n=0; n<Nd_; ++n) 
+      os << Re(u_[n][ny]) <<sp<< Im(u_[n][ny]) <<sp;
+    os << '\n';
+  }
   os.close();
 
   ((BasisFunc&) *this).makeState(origstate);
 }
 
-void BasisFunc::m4save(const string& filebase) const {
-  string filename = filebase + string(".asc");
-  ofstream os(filename.c_str());
-  os << setprecision(REAL_DIGITS);
-  const int w = REAL_IOWIDTH;
-  const char s=' ';
-  for (int i=0; i<Ny_; ++i) 
-    os << setw(w) << Re(u_[i]) <<s<< setw(w) << Im(u_[i]) <<s
-       << setw(w) << Re(v_[i]) <<s<< setw(w) << Im(v_[i]) <<s
-       << setw(w) << Re(w_[i]) <<s<< setw(w) << Im(w_[i]) <<'\n';
-  os.close();
-}
-
 void BasisFunc::binaryDump(ostream& os) const {
+  write(os, Nd_);
   write(os, Ny_);
   write(os, kx_);
   write(os, kz_);
@@ -231,9 +241,8 @@ void BasisFunc::binaryDump(ostream& os) const {
   write(os, b_);
   write(os, state_);
   write(os, lambda_);
-  u_.binaryDump(os);
-  v_.binaryDump(os);
-  w_.binaryDump(os);
+  for (int n=0; n<Nd_; ++n)
+    u_[n].binaryDump(os);
 }
 
 void BasisFunc::binaryLoad(istream& is) {
@@ -250,13 +259,13 @@ void BasisFunc::binaryLoad(istream& is) {
   read(is, b_);
   read(is, state_);
   read(is, lambda_);
-  u_.binaryLoad(is);
-  v_.binaryLoad(is);
-  w_.binaryLoad(is);
+  for (int n=0; n<Nd_; ++n)
+    u_[n].binaryLoad(is);
 }
   
 void BasisFunc::randomize(bool adiri, bool bdiri, Real decay) {
-  
+  assert(Nd_ == 3);
+
   setToZero();
   setState(Spectral);
 
@@ -277,8 +286,8 @@ void BasisFunc::randomize(bool adiri, bool bdiri, Real decay) {
     diff(v, u);
     u *= I*Lx_/(2*pi*kx_);
   }
-  u_ += u;
-  v_ += v;
+  u_[0] += u;
+  u_[1] += v;
 
   u.setToZero();
   v.setToZero();
@@ -295,51 +304,48 @@ void BasisFunc::randomize(bool adiri, bool bdiri, Real decay) {
     diff(v, w);
     w *= I*Lz_/(2*pi*kz_);
   }
-  w_ += w;
-  v_ += v;
+  u_[2] += w;
+  u_[1] += v;
   
   if (kx_ == 0 && kz_ == 0) {
-    u_.im.setToZero();
-    w_.im.setToZero();
-    v_.re.setToZero(); // should already be zero
+    u_[0].im.setToZero();
+    u_[1].re.setToZero(); // should already be zero
+    u_[2].im.setToZero();
   }
 }
+
 void BasisFunc::interpolate(const BasisFunc& phi) {
-  assert(a_ >= phi.a_ && b_ <= phi.b_);
   assert(phi.state_ == Spectral);
-  u_.interpolate(phi.u_);
-  v_.interpolate(phi.v_);
-  w_.interpolate(phi.w_);
+  assert(a_ >= phi.a_ && b_ <= phi.b_);
+  assert(Nd_ == phi.Nd_);
+  for (int n=0; n<Nd_; ++n)
+    u_[n].interpolate(phi.u_[n]);
   state_ = Physical;
 }
 
 void BasisFunc::reflect(const BasisFunc& phi) {
   assert((a_+b_)/2 == phi.a() && b_ <= phi.b() && b_ > phi.a());
   assert(phi.state_ == Spectral);
-  u_.reflect(phi.u_, Odd);
-  v_.reflect(phi.v_, Even);
-  w_.reflect(phi.w_, Odd);
-  //ChebyCoeff tmp = v_.re;
-  //v_.re = v_.im;
-  //v_.im = tmp;
+  for (int n=0; n<Nd_; n += 2)
+    u_[n].reflect(phi.u_[n], Odd);
+  for (int n=1; n<Nd_; n += 2)
+    u_[n].reflect(phi.u_[n], Even);
   state_ = Physical;
 }
   
 void BasisFunc::resize(int N) {
   if (N != Ny_) {
     Ny_ = N;
-    u_.resize(Ny_);
-    v_.resize(Ny_);
-    w_.resize(Ny_);
+    for (int n=0; n<Nd_; ++n)
+      u_[n].resize(Ny_);
   }
 }
 
 void BasisFunc::setBounds(Real Lx, Real Lz, Real a, Real b) {
   Lx_ = Lx;
   Lz_ = Lz;
-  u_.setBounds(a,b);
-  v_.setBounds(a,b);
-  w_.setBounds(a,b);
+  for (int n=0; n<Nd_; ++n)
+    u_[n].setBounds(a,b);
 }
 void BasisFunc::setkxkz(int kx, int kz) {
   kx_=kx;
@@ -347,9 +353,8 @@ void BasisFunc::setkxkz(int kx, int kz) {
 }
 void BasisFunc::setState(fieldstate s) {
   assert(s == Spectral || s == Physical);
-  u_.setState(s);
-  v_.setState(s);
-  w_.setState(s);
+  for (int n=0; n<Nd_; ++n)
+    u_[n].setState(s);
   state_ = s;
 }
 
@@ -358,35 +363,34 @@ void BasisFunc::setRmsval(Real lambda) {
 }
 
 void BasisFunc::setToZero() {
-   u_.setToZero();
-   v_.setToZero();
-   w_.setToZero();
+  for (int n=0; n<Nd_; ++n)
+    u_[n].setToZero();
 }
 
 void BasisFunc::fill(const BasisFunc& f) {
-   u_.fill(f.u_);
-   v_.fill(f.v_);
-   w_.fill(f.w_);
+  for (int n=0; n<Nd_; ++n)
+    u_[n].fill(f.u_[n]);
 }
 void BasisFunc::conjugate() {
-  u_.conjugate();
-  v_.conjugate();
-  w_.conjugate();
+  for (int n=0; n<Nd_; ++n)
+    u_[n].conjugate();
   kx_ *= -1;
   kz_ *= -1; // Note that no neg vals of kz appear in FlowFields.
 }
 
 Real BasisFunc::bcNorm(bool a_dirichlet, bool b_dirichlet) const {
   Real err=0.0;
-  if (b_dirichlet) 
-    err += abs2(u_.eval_b()) + abs2(v_.eval_b()) + abs2(w_.eval_b());
   if (a_dirichlet)
-    err += abs2(u_.eval_a()) + abs2(v_.eval_a()) + abs2(w_.eval_a());
+    for (int n=0; n<Nd_; ++n)
+      err += abs2(u_[n].eval_a());
+  if (b_dirichlet) 
+    for (int n=0; n<Nd_; ++n)
+      err += abs2(u_[n].eval_b());
   return sqrt(err);
 }
 
 bool BasisFunc::geometryCongruent(const BasisFunc& Phi) const {
-  return (Ny_==Phi.Ny_ && Lx_==Phi.Lx_ && Lz_==Phi.Lz_ &&
+  return (Nd_ == Phi.Nd_ && Ny_==Phi.Ny_ && Lx_==Phi.Lx_ && Lz_==Phi.Lz_ &&
 	  a_==Phi.a_ && b_==Phi.b_);
 }
 
@@ -398,145 +402,89 @@ bool BasisFunc::congruent(const BasisFunc& Phi) const {
 void BasisFunc::chebyfft() {
   assert(state_ == Physical);
   ChebyTransform t(Ny_);
-  t.chebyfft(u_);
-  t.chebyfft(v_);
-  t.chebyfft(w_);
+  for (int n=0; n<Nd_; ++n)
+    u_[n].chebyfft(t);
   state_ = Spectral;
 }
 
 void BasisFunc::ichebyfft() {
   assert(state_ == Spectral);
   ChebyTransform t(Ny_);
-  t.ichebyfft(u_);
-  t.ichebyfft(v_);
-  t.ichebyfft(w_);
+  for (int n=0; n<Nd_; ++n)
+    u_[n].ichebyfft(t);
   state_ = Physical;
 }
 void BasisFunc::makeSpectral()  {
   ChebyTransform t(Ny_);
-  t.makeSpectral(u_);
-  t.makeSpectral(v_);
-  t.makeSpectral(w_);
+  for (int n=0; n<Nd_; ++n)
+    u_[n].makeSpectral(t);
   state_ = Spectral;
 }
 void BasisFunc::makePhysical() {
   ChebyTransform t(Ny_);
-  t.makePhysical(u_);
-  t.makePhysical(v_);
-  t.makePhysical(w_);
+  for (int n=0; n<Nd_; ++n)
+    u_[n].makePhysical(t);
   state_ = Physical;
 }
 void BasisFunc::makeState(fieldstate s) {
-  if (state_ != s) {
-    ChebyTransform t(Ny_);
-    if (s == Physical) {
-      t.makePhysical(u_);
-      t.makePhysical(v_);
-      t.makePhysical(w_);
-      state_ = Physical;
-    }
-    else {
-      t.makeSpectral(u_);
-      t.makeSpectral(v_);
-      t.makeSpectral(w_);
-      state_ = Spectral;
-    }
-  }
+  ChebyTransform t(Ny_);
+  for (int n=0; n<Nd_; ++n)
+    u_[n].makeState(s,t);
+  state_ = s;
 }
 
 void BasisFunc::chebyfft(const ChebyTransform& t) {
   assert(state_ == Physical);
-  t.chebyfft(u_);
-  t.chebyfft(v_);
-  t.chebyfft(w_);
+  for (int n=0; n<Nd_; ++n)
+    u_[n].chebyfft(t);
   state_ = Spectral;
 }
 
 void BasisFunc::ichebyfft(const ChebyTransform& t) {
   assert(state_ == Spectral);
-  t.ichebyfft(u_);
-  t.ichebyfft(v_);
-  t.ichebyfft(w_);
+  for (int n=0; n<Nd_; ++n)
+    u_[n].ichebyfft(t);
   state_ = Physical;
 }
 void BasisFunc::makeSpectral(const ChebyTransform& t)  {
-  t.makeSpectral(u_);
-  t.makeSpectral(v_);
-  t.makeSpectral(w_);
+  for (int n=0; n<Nd_; ++n)
+    u_[n].makeSpectral(t);
   state_ = Spectral;
 }
 void BasisFunc::makePhysical(const ChebyTransform& t) {
-  t.makePhysical(u_);
-  t.makePhysical(v_);
-  t.makePhysical(w_);
+  for (int n=0; n<Nd_; ++n)
+    u_[n].makePhysical(t);
   state_ = Physical;
 }
 void BasisFunc::makeState(fieldstate s, const ChebyTransform& t) {
-  if (s == Physical) {
-    t.makePhysical(u_);
-    t.makePhysical(v_);
-    t.makePhysical(w_);
-    state_ = Physical;
-  }
-  else {
-    t.makeSpectral(u_);
-    t.makeSpectral(v_);
-    t.makeSpectral(w_);
-    state_ = Spectral;
-  }
+  for (int n=0; n<Nd_; ++n)
+    u_[n].makeState(s,t);
+  state_ = s;
 }
 
 
-const ComplexChebyCoeff& BasisFunc::u() const {return u_;}
-const ComplexChebyCoeff& BasisFunc::v() const {return v_;}
-const ComplexChebyCoeff& BasisFunc::w() const {return w_;}
-ComplexChebyCoeff& BasisFunc::u() {return u_;}
-ComplexChebyCoeff& BasisFunc::v() {return v_;}
-ComplexChebyCoeff& BasisFunc::w() {return w_;}
+const ComplexChebyCoeff& BasisFunc::u() const {return u_[0];}
+const ComplexChebyCoeff& BasisFunc::v() const {return u_[1];}
+const ComplexChebyCoeff& BasisFunc::w() const {return u_[2];}
+ComplexChebyCoeff& BasisFunc::u() {return u_[0];}
+ComplexChebyCoeff& BasisFunc::v() {return u_[1];}
+ComplexChebyCoeff& BasisFunc::w() {return u_[2];}
 
 const ComplexChebyCoeff& BasisFunc::component(int i) const {
-  assert(i>=0 && i<=2);
-  switch(i) {
-  case (0): return u_;
-  case (1): return v_;
-  case (2): return w_;
-  default: 
-    exit(-1); // so the compiler doesn't complain
-    return w_;
-  }
+  assert(i>=0 && i<Nd_);
+  return u_[i];
 }
 ComplexChebyCoeff& BasisFunc::component(int i) {
-  assert(i>=0 && i<=2);
-  switch(i) {
-  case (0): return u_;
-  case (1): return v_;
-  case (2): return w_;
-  default: 
-    exit(-1);
-    return w_; // so the compiler doesn't complain
-  }
+  assert(i>=0 && i<Nd_);
+  return u_[i];
 }
 const ComplexChebyCoeff& BasisFunc::operator[](int i) const {
-  assert(i>=0 && i<=2);
-  switch(i) {
-  case (0): return u_;
-  case (1): return v_;
-  case (2): return w_;
-  default: 
-    exit(-1); // so the compiler doesn't complain
-    return w_;
-  }
+  assert(i>=0 && i<Nd_);
+  return u_[i];
 }
 ComplexChebyCoeff& BasisFunc::operator[](int i) {
-  assert(i>=0 && i<=2);
-  switch(i) {
-  case (0): return u_;
-  case (1): return v_;
-  case (2): return w_;
-  default: 
-    exit(-1);
-    return w_; // so the compiler doesn't complain
-  }
+  assert(i>=0 && i<Nd_);
+  return u_[i];
 }
 /**************
 bool BasisFunc::operator==(const BasisFunc& Phi) const {
@@ -547,63 +495,64 @@ bool BasisFunc::operator!=(const BasisFunc& Phi) const {
 }
 ****************/
 
-bool BasisFunc::operator==(const BasisFunc& Phi) const {
-  return (congruent(Phi) && u_==Phi.u_ && v_==Phi.v_ && w_==Phi.w_) 
-    ? true : false;
-}
-bool BasisFunc::operator!=(const BasisFunc& Phi) const {
-  return (*this == Phi) ? false : true;
-}
-
 BasisFunc& BasisFunc::operator *= (const BasisFunc& phi) {
-  u_ *= phi.u_;
-  v_ *= phi.v_;
-  w_ *= phi.w_;
+  for (int n=0; n<Nd_; ++n)
+    u_[n] *= phi.u_[n];
   return *this;
 }
 
 BasisFunc& BasisFunc::operator += (const BasisFunc& phi) {
-  u_ += phi.u_;
-  v_ += phi.v_;
-  w_ += phi.w_;
+  for (int n=0; n<Nd_; ++n)
+    u_[n] += phi.u_[n];
   return *this;
 }
 BasisFunc& BasisFunc::operator -= (const BasisFunc& phi) {
-  u_ -= phi.u_;
-  v_ -= phi.v_;
-  w_ -= phi.w_;
+  for (int n=0; n<Nd_; ++n)
+    u_[n] -= phi.u_[n];
   return *this;
 }
 BasisFunc& BasisFunc::operator *= (Real c) {
-  u_ *= c;
-  v_ *= c;
-  w_ *= c;
+  for (int n=0; n<Nd_; ++n)
+    u_[n] *= c;
   return *this;
 }
 BasisFunc& BasisFunc::operator *= (Complex c) {
-  u_ *= c;
-  v_ *= c;
-  w_ *= c;
+  for (int n=0; n<Nd_; ++n)
+    u_[n] *= c;
   return *this;
 }
+
+bool operator!=(const BasisFunc& f, const BasisFunc& g) {
+  return (f == g) ? false : true;
+}
+
+bool operator==(const BasisFunc& f, const BasisFunc& g) {
+  if (!f.congruent(g))
+    return false;
+
+  for (int n=0; n<f.Nd(); ++n)
+    if (f[n] != g[n])
+      return false;
+
+  return true;
+}
+
    
 // ==================================================================
 // L2 norms
 Real L2Norm(const BasisFunc& phi, bool normalize) {
-  Real rtn2 = 
-    L2Norm2(phi.u(), normalize) +
-    L2Norm2(phi.v(), normalize) + 
-    L2Norm2(phi.w(), normalize);
+  Real rtn2 = 0.0;
+  for (int n=0; n<phi.Nd(); ++n)
+    rtn2 += L2Norm2(phi[n], normalize);
   if (!normalize)
     rtn2 *= phi.Lx()*phi.Lz();
   return sqrt(rtn2);
 }
 
 Real L2Norm2(const BasisFunc& phi, bool normalize) {
-  Real rtn = 
-    L2Norm2(phi.u(), normalize) +
-    L2Norm2(phi.v(), normalize) + 
-    L2Norm2(phi.w(), normalize);
+  Real rtn = 0.0;
+  for (int n=0; n<phi.Nd(); ++n)
+    rtn += L2Norm2(phi[n], normalize);
   if (!normalize)
     rtn *= phi.Lx()*phi.Lz();
   return rtn;
@@ -613,10 +562,8 @@ Real L2Dist2(const BasisFunc& f, const BasisFunc& g, bool normalize) {
   assert(f.geometryCongruent(g));
   Real rtn=0.0;
   if (f.kx()==g.kx() && f.kz()==g.kz())
-    rtn = 
-      L2Dist2(f.u(), g.u(), normalize) +
-      L2Dist2(f.v(), g.v(), normalize) +
-      L2Dist2(f.w(), g.w(), normalize);
+    for (int n=0; n<f.Nd(); ++n)
+      rtn += L2Dist2(f[n], g[n], normalize);
   if (!normalize)
     rtn *= f.Lx()*f.Lz();
   return rtn;
@@ -632,10 +579,8 @@ Complex L2InnerProduct(const BasisFunc& f, const BasisFunc& g,
   assert(f.geometryCongruent(g));
   Complex rtn(0.0, 0.0);
   if (f.kx()==g.kx() && f.kz()==g.kz())
-    rtn = 
-      L2InnerProduct(f.u(), g.u(), normalize) +
-      L2InnerProduct(f.v(), g.v(), normalize) +
-      L2InnerProduct(f.w(), g.w(), normalize);
+    for (int n=0; n<f.Nd(); ++n)
+      rtn += L2InnerProduct(f[n], g[n], normalize);
   if (!normalize)
     rtn *= f.Lx() * f.Lz();
   return rtn;
@@ -644,20 +589,18 @@ Complex L2InnerProduct(const BasisFunc& f, const BasisFunc& g,
 // ==================================================================
 // cheby norms
 Real chebyNorm(const BasisFunc& phi, bool normalize) {
-  Real rtn2 = 
-    chebyNorm2(phi.u(), normalize) +
-    chebyNorm2(phi.v(), normalize) + 
-    chebyNorm2(phi.w(), normalize);
+  Real rtn2 = 0.0;
+  for (int n=0; n<phi.Nd(); ++n)
+    rtn2 += chebyNorm2(phi[n], normalize);
   if (!normalize)
     rtn2 *= phi.Lx()*phi.Lz();
   return sqrt(rtn2);
 }
 
 Real chebyNorm2(const BasisFunc& phi, bool normalize) {
-  Real rtn = 
-    chebyNorm2(phi.u(), normalize) +
-    chebyNorm2(phi.v(), normalize) + 
-    chebyNorm2(phi.w(), normalize);
+  Real rtn = 0.0;
+  for (int n=0; n<phi.Nd(); ++n)
+    rtn += chebyNorm2(phi[n], normalize);
   if (!normalize)
     rtn *= phi.Lx()*phi.Lz();
   return rtn;
@@ -667,10 +610,8 @@ Real chebyDist2(const BasisFunc& f, const BasisFunc& g, bool normalize) {
   assert(f.geometryCongruent(g));
   Real rtn=0.0;
   if (f.kx()==g.kx() && f.kz()==g.kz())
-    rtn = 
-      chebyDist2(f.u(), g.u(), normalize) +
-      chebyDist2(f.v(), g.v(), normalize) +
-      chebyDist2(f.w(), g.w(), normalize);
+    for (int n=0; n<f.Nd(); ++n)
+      rtn += chebyDist2(f[n], g[n], normalize);
   if (!normalize)
     rtn *= f.Lx()*f.Lz();
   return rtn;
@@ -686,10 +627,8 @@ Complex chebyInnerProduct(const BasisFunc& f, const BasisFunc& g,
   assert(f.geometryCongruent(g));
   Complex rtn(0.0, 0.0);
   if (f.kx()==g.kx() && f.kz()==g.kz())
-    rtn = 
-      chebyInnerProduct(f.u(), g.u(), normalize) +
-      chebyInnerProduct(f.v(), g.v(), normalize) +
-      chebyInnerProduct(f.w(), g.w(), normalize);
+    for (int n=0; n<f.Nd(); ++n)
+      rtn += chebyInnerProduct(f[n], g[n], normalize);
   if (!normalize)
     rtn *= f.Lx() * f.Lz();
   return rtn;
