@@ -272,13 +272,22 @@ void ChebyCoeff::makeState(fieldstate s){
 void ChebyCoeff::chebyfft(const ChebyTransform& t) {
   assert(t.N() == N_);
   assert(state_ == Physical);
+  if (N_ < 2) {
+    state_ = Spectral;
+    return;
+  }
 
   fftw_execute_r2r(t.cosfftw_plan_, data_, data_);
 
-  Real nrm = 1.0/(N_-1); // needed because FFTW does unnormalized transforms
+  // Factors of 1/2 appear on 0th and (N-1)th coeff because of reln btwn
+  // chebyshev and cosine transform. Normalization factor nrm=1/(N-1) is 
+  // needed because FFTW does unnormalized transforms. After this loop,
+  // data_[n] equals the coefficient of T_n.
+  Real nrm = 1.0/(N_-1); 
   data_[0] *= 0.5*nrm;
-  for (int n=1; n<N_; ++n)
+  for (int n=1; n<N_-1; ++n)
     data_[n] *= nrm;
+  data_[N_-1] *= 0.5*nrm;
   
   state_ = Spectral;
   return;
@@ -287,9 +296,14 @@ void ChebyCoeff::chebyfft(const ChebyTransform& t) {
 void ChebyCoeff::ichebyfft(const ChebyTransform& t){
   assert(t.N() == N_);
   assert(state_ == Spectral);
+  if (N_ < 2) {
+    state_ = Physical;
+    return;
+  }
 
-  // 0th elem is different due to reln between cheby and cos transforms.
-  data_[0] *= 2.0;
+  // Factors of 2 undo the 1/2's introduced in ::chebyfft.
+  data_[0]    *= 2.0;
+  data_[N_-1] *= 2.0;
   fftw_execute_r2r(t.cosfftw_plan_, data_, data_);
   for (int n=0; n<N_; ++n)
     data_[n] *= 0.5;
@@ -323,14 +337,15 @@ void ChebyCoeff::randomize(Real decay, bool a_dirichlet, bool b_dirichlet) {
     data_[n] = pow_decay_n * randomReal();
     pow_decay_n *= decay;
   }
-  
-  if (a_dirichlet && b_dirichlet) {
+  if (N_ == 1 && (a_dirichlet || b_dirichlet))
+    data_[0] = 0.0;
+  else if (N_ >= 2 && (a_dirichlet && b_dirichlet)) {
     data_[1] -= 0.5*(eval_b() - eval_a());
     data_[0] -= 0.5*(eval_b() + eval_a());
   }
-  else if (a_dirichlet)
+  else if (N_ >= 2 && a_dirichlet)
     data_[0] -= eval_a();
-  else if (a_dirichlet)
+  else if (N_ >= 2 && b_dirichlet)
     data_[0] -= eval_b();
 
   makeState(startState);
@@ -398,6 +413,8 @@ void ChebyCoeff::setState(fieldstate s) {
 
 
 Real ChebyCoeff::eval_b() const {
+  if (N_==0)
+    return NAN;
   if (state_ == Spectral) {
     Real sum = 0.0;
     for (int n=N_-1; n>=0; --n)
@@ -409,18 +426,22 @@ Real ChebyCoeff::eval_b() const {
 }
 
 Real ChebyCoeff::eval_a()  const {
+  if (N_==0)
+    return NAN;
   if (state_ == Spectral) {
-    Real sum = 0.0;
+    Real sum=0.0;
     for (int n=N_-1; n>=0; --n) 
       sum += data_[n] * ((n%2 == 0) ? 1 : -1);
     return sum;
   }
-  else
+  else 
     return data_[N_-1];
 }
 
 Real ChebyCoeff::eval(Real x) const {
   assert(state_ == Spectral);
+  if (N_==0)
+    return NAN;
   Real y = (2*x-a_-b_)/(b_-a_);
   Real y2 = 2*y;
   Real d=0.0;
