@@ -1,13 +1,16 @@
 /* diffops.cpp: Differential operators for FlowFields
  * Channelflow-0.9
  *
- * Copyright (C) 2001-2005  John F. Gibson  
- *  
- * jgibson@mail.sjcsf.edu  
- * John F. Gibson 
- * St. John's College
- * 1160 Camino de la Cruz Blanca
- * Santa Fe, NM 87501
+ * Copyright (C) 2001-2005  John F. Gibson
+ *
+ * John F. Gibson
+ * Center for Nonlinear Sciences
+ * School of Physics
+ * Georgia Institute of Technology
+ * Atlanta, GA 30332-0430
+ *
+ * gibson@cns.physics.gatech.edu
+ * jfg@member.fsf.org
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,8 +27,10 @@
  */
 
 #include <fstream>
+#include <iomanip>
 #include "diffops.h"
 
+
 void xdiff(const FlowField& f_, FlowField& dfdx, int n) {
   if (n==0)
     return;
@@ -56,14 +61,17 @@ void xdiff(const FlowField& f_, FlowField& dfdx, int n) {
   int Mx = f.Mx();
   int My = f.My();
   int Mz = f.Mz();
+  int kxmax = f.kxmax();
+
   for (int i=0; i<Nd; ++i)
     for (int my=0; my<My; ++my)
       for (int mx=0; mx<Mx; ++mx) {
 	int kx = f.kx(mx);
-	Complex cx = rot*pow(2*pi*kx/Lx, n);
+	Complex cx = rot*(pow(2*pi*kx/Lx, n)*zero_last_mode(kx,kxmax,n));
 	for (int mz=0; mz<Mz; ++mz) 
 	  dfdx.cmplx(mx,my,mz,i) = cx*f.cmplx(mx,my,mz,i);
       }
+
   f.makeState(sxz,sy);
   dfdx.makeSpectral();
   return;
@@ -145,8 +153,6 @@ void zdiff(const FlowField& f_, FlowField& dfdz, int n) {
   fieldstate sy  = f.ystate();
   f.makeSpectral_xz();
 
-  Real Lz = f.Lz();
-
   // compute gradf(nx,ny,nz,i) = df(nx,ny,nz,0)/dx_i for scalar-valued f
   if (!f.congruent(dfdz))
     dfdz.resize(f.Nx(), f.Ny(), f.Nz(), f.Nd(), f.Lx(), f.Lz(), f.a(), f.b());
@@ -167,13 +173,16 @@ void zdiff(const FlowField& f_, FlowField& dfdz, int n) {
   int Mx = f.Mx();
   int My = f.My();
   int Mz = f.Mz();
+  int kzmax = f.kzmax();
+  Real Lz = f.Lz();
 
   Real az = 2*pi/Lz;
   for (int i=0; i<Nd; ++i)
     for (int my=0; my<My; ++my)
       for (int mx=0; mx<Mx; ++mx) 
 	for (int mz=0; mz<Mz; ++mz) {
-	  Complex cz = rot*pow(az*f.kz(mz), n);
+	  int kz = f.kz(mz);
+	  Complex cz = rot*(pow(az*kz, n)*zero_last_mode(kz,kzmax,n));
 	  dfdz.cmplx(mx,my,mz,i) = cz*f.cmplx(mx,my,mz,i);
 	}
 
@@ -218,16 +227,20 @@ void diff(const FlowField& f_, FlowField& df, int nx, int ny, int nz) {
   int Mx = f.Mx();
   int My = f.My();
   int Mz = f.Mz();
-  
+  int kxmax = f.kxmax();
+  int kzmax = f.kzmax();
+
   // Do the x and z differentiation
   Real ax = 2*pi/Lx;
   Real az = 2*pi/Lz;
   for (int i=0; i<Nd; ++i)
     for (int my=0; my<My; ++my)
       for (int mx=0; mx<Mx; ++mx) {
-	Complex cx = xrot*pow(ax*f.kx(mx), nx);
+	int kx = f.kx(mx);
+	Complex cx = xrot*(pow(ax*kx, nx)*zero_last_mode(kx,kxmax,nx));
 	for (int mz=0; mz<Mz; ++mz) {
-	  Complex cz = zrot*pow(az*f.kz(mz), nz);
+	  int kz = f.kz(mz);
+	  Complex cz = zrot*(pow(az*kz, nz)*zero_last_mode(kz,kzmax,nz));
 	  df.cmplx(mx,my,mz,i) = cx*cz*f.cmplx(mx,my,mz,i);
 	}
       }
@@ -280,17 +293,20 @@ void grad(const FlowField& f_, FlowField& gradf) {
     int Mx = f.Mx();
     int My = f.My();
     int Mz = f.Mz();
-    
+    int kxmax = f.kxmax();
+    int kzmax = f.kzmax();
+
     // Assign df/dx to Df(i) and df/dz to Df(i)
-    Real cz = 2.0*pi/Lz;
     for (int my=0; my<My; ++my)
       for (int mx=0; mx<Mx; ++mx) {
 	int kx = f.kx(mx);
-	Complex cxkx(0.0, 2.0*pi*kx/Lx);
+	Complex cxkx(0.0, 2.0*pi*kx/Lx*zero_last_mode(kx,kxmax,1));
 	for (int mz=0; mz<Mz; ++mz) {
+	  int kz = f.kz(mz);
+	  Complex czkz(0.0, 2.0*pi*kz/Lz*zero_last_mode(kz,kzmax,1));
 	  Complex fval = f.cmplx(mx,my,mz,0);
 	  gradf.cmplx(mx,my,mz,0) = fval*cxkx;
-	  gradf.cmplx(mx,my,mz,2) = fval*Complex(0.0, f.kz(mz)*cz);
+	  gradf.cmplx(mx,my,mz,2) = fval*czkz;
 	}
       }
 
@@ -317,17 +333,21 @@ void grad(const FlowField& f_, FlowField& gradf) {
     int My = f.My();
     int Mz = f.Mz();
     int Nd = 3;
+    int kxmax = f.kxmax();
+    int kzmax = f.kzmax();
   
     // Assign df_i/dx to Df(i,0) and df_i/dz to Df(i,2)
-    Real cz = 2.0*pi/Lz;
     for (int i=0; i<Nd; ++i)
       for (int my=0; my<My; ++my)
 	for (int mx=0; mx<Mx; ++mx) {
-	  Complex cxkx = (2.0*pi*f.kx(mx)/Lx)*I;
+	  int kx = f.kx(mx);
+	  Complex cxkx(0.0, 2.0*pi*kx/Lx*zero_last_mode(kx,kxmax,1));
 	  for (int mz=0; mz<Mz; ++mz) {
+	    int kz = f.kz(mz);
+	    Complex czkz(0.0, 2.0*pi*kz/Lz*zero_last_mode(kz,kzmax,1));
 	    Complex fval = f.cmplx(mx,my,mz,i);
 	    gradf.cmplx(mx,my,mz,i3j(i,0)) = fval*cxkx;
-	    gradf.cmplx(mx,my,mz,i3j(i,2)) = fval*Complex(0.0, cz*f.kz(mz));
+	    gradf.cmplx(mx,my,mz,i3j(i,2)) = fval*czkz;
 	  }
 	}
 
@@ -477,6 +497,8 @@ void curl(const FlowField& f_, FlowField& curlf) {
   int Mx = f.Mx();
   int My = f.My();
   int Mz = f.Mz();
+  int kxmax = f.kxmax();
+  int kzmax = f.kzmax();
   Real Lx = f.Lx();
   Real Lz = f.Lz();
   
@@ -487,9 +509,11 @@ void curl(const FlowField& f_, FlowField& curlf) {
   // Assign d/dx and d/dz terms to curl
   for (int my=0; my<My; ++my)
     for (int mx=0; mx<Mx; ++mx) {
-      Complex cx = (2.0*pi*f.kx(mx)/Lx)*I;
+      int kx = f.kx(mx);
+      Complex cx(0.0, 2.0*pi*kx/Lx*zero_last_mode(kx,kxmax,1));
       for (int mz=0; mz<Mz; ++mz) {
-	Complex cz = (2.0*pi*f.kz(mz)/Lz)*I;
+	int kz = f.kz(mz);
+	Complex cz(0.0, 2.0*pi*kz/Lz*zero_last_mode(kz,kzmax,1));
 	Complex u = f.cmplx(mx,my,mz,0);
 	Complex v = f.cmplx(mx,my,mz,1);
 	Complex w = f.cmplx(mx,my,mz,2);
@@ -603,6 +627,8 @@ void div(const FlowField& f_,  FlowField& divf) {
   int Mx = f.Mx();
   int My = f.My();
   int Mz = f.Mz();
+  int kxmax = f.kxmax();
+  int kzmax = f.kzmax();
   Real Lx = f.Lx();
   Real Lz = f.Lz();
 
@@ -614,14 +640,16 @@ void div(const FlowField& f_,  FlowField& divf) {
     divf.setState(Spectral, Spectral);
     
     // Add df0/dx + df2/dz to divf
-    Real cz = 2.0*pi/Lz;
     for (int my=0; my<My; ++my)
       for (int mx=0; mx<Mx; ++mx) {
-	Complex cxkx(0.0, 2.0*pi*f.kx(mx)/Lx);
-	for (int mz=0; mz<Mz; ++mz) 
+	int kx = f.kx(mx);
+	Complex cxkx(0.0, 2.0*pi*kx/Lx*zero_last_mode(kx,kxmax,1));
+	for (int mz=0; mz<Mz; ++mz) {
+	  int kz = f.kz(mz);
+	  Complex czkz(0.0, 2.0*pi*kz/Lz*zero_last_mode(kz,kzmax,1));
 	  divf.cmplx(mx,my,mz,0) 
-	    = f.cmplx(mx,my,mz,0)*cxkx 
-	    + f.cmplx(mx,my,mz,2)*Complex(0.0, cz*f.kz(mz));
+	    = f.cmplx(mx,my,mz,0)*cxkx + f.cmplx(mx,my,mz,2)*czkz;
+	}
       }
 
     // Add df1/dy to to divf
@@ -646,14 +674,17 @@ void div(const FlowField& f_,  FlowField& divf) {
     for (int j=0; j<3; ++j) {
 
       // Add df0j/dx + df2j/dz to divfj
-      Real cz = 2.0*pi/Lz;
       for (int my=0; my<My; ++my)
 	for (int mx=0; mx<Mx; ++mx) {
-	  Complex cxkx(0.0, 2.0*pi*f.kx(mx)/Lx);
-	    for (int mz=0; mz<Mz; ++mz) 
+	  int kx = f.kx(mx);
+	  Complex cxkx(0.0, 2.0*pi*f.kx(mx)/Lx*zero_last_mode(kx,kxmax,1));
+	  for (int mz=0; mz<Mz; ++mz) {
+	    int kz = f.kz(mz);
+	    Complex czkz(0.0, 2.0*pi*kz/Lz*zero_last_mode(kz,kzmax,1));
 	      divf.cmplx(mx,my,mz,j) +=
 		f.cmplx(mx,my,mz,i3j(0,j))*cxkx + 
-		f.cmplx(mx,my,mz,i3j(2,j))*Complex(0.0, cz*f.kz(mz));
+		f.cmplx(mx,my,mz,i3j(2,j))*czkz;
+	  }
 	}
 
       // Add df1j/dy to to divf
@@ -833,36 +864,6 @@ FlowField energy(const FlowField& u, ChebyCoeff& U) {
   FlowField e; energy(u,U,e); return e;
 }
     
-void matlabSave(FlowField& f, int i, string& filename) {
-  fieldstate initial_xzstate = f.xzstate();
-  fieldstate initial_ystate = f.ystate();
-  f.makePhysical();
-
-  string var=filename;
-  filename = filename + ".m";
-
-  ofstream ofs(filename.c_str());
-  ofs.precision(10);
-
-  for(long ny=0; ny<f.Ny(); ++ny) {
-    ofs << var << "(:,:," << (ny+1) << ")= [";
-    for (int nx=0; nx<f.Nx(); ++nx) {
-      for (int nz=0; nz<f.Nz(); ++nz) {
-	ofs << f(nx, ny, nz, i) << ' ';
-      }
-      ofs << ";";
-    }
-    ofs << "];";
-  }
-  ofs.close();
-  if (initial_xzstate == Spectral) {
-    f.realfft_xz();
-  }
-  if (initial_ystate == Spectral) {
-    f.chebyfft_y();
-  }
-}
-      
 void navierstokesNL(const FlowField& u,const ChebyCoeff& U, FlowField& f, 
 		    FlowField& tmp, NonlinearMethod& method) {
   switch (method) {
@@ -946,12 +947,14 @@ void rotationalNL(const FlowField& u_, const ChebyCoeff& U_, FlowField& f,
   int Mx = u.Mx();
   int My = u.My();
   int Mz = u.Mz();
+  int kxmax = f.kxmax();
 
   for (int ny=0; ny<My; ++ny) { 
     Real rU  = U[ny];
     Real rUy = Uy[ny];
     for (int mx=0; mx<Mx; ++mx) {
-      Complex d_dx(0.0, 2*pi*u.kx(mx)/u.Lx());
+      int kx = u.kx(mx);
+      Complex d_dx(0.0, 2*pi*kx/u.Lx()*zero_last_mode(kx,kxmax,1));
       for (int mz=0; mz<Mz; ++mz) {
 	uc = u.cmplx(mx,ny,mz, 0);
 	vc = u.cmplx(mx,ny,mz, 1);
@@ -1148,7 +1151,7 @@ void skewsymmetricNL(const FlowField& u_, const ChebyCoeff& U_, FlowField& f, Fl
   int Nx = u.Nx();
   int Ny = u.Ny();
   int Nz = u.Nz();
-
+  
   // Accumulate 1/2 u_j du_i/dx_j in f_i
   for (int i=0; i<3; ++i) 
     for (int j=0; j<3; ++j) {
@@ -1189,9 +1192,10 @@ void skewsymmetricNL(const FlowField& u_, const ChebyCoeff& U_, FlowField& f, Fl
   int Mx = u.Mx();
   int My = u.My();
   int Mz = u.Mz();
-
-  Real cx = 2*pi/u.Lx();
-  Real cz = 2*pi/u.Lz();
+  int kxmax = u.kxmax();
+  int kzmax = u.kzmax();
+  Real Lx = u.Lx();
+  Real Lz = u.Lz();
 
   ComplexChebyCoeff tmpProfile(My, u.a(), u.b(), Spectral);
   ComplexChebyCoeff tmpProfile_y(My, u.a(), u.b(), Spectral);
@@ -1205,9 +1209,11 @@ void skewsymmetricNL(const FlowField& u_, const ChebyCoeff& U_, FlowField& f, Fl
     // Add in du_i/dx and du_i/dz, that is, d/dx_j (u_i u_j) for j=0,2
     for (int my=0; my<My; ++my)     
       for (int mx=0; mx<Mx; ++mx) {
-	Complex d_dx(0.0, cx*u.kx(mx));
+	int kx = u.kx(mx);
+	Complex d_dx(0.0, 2*pi*kx/Lx*zero_last_mode(kx,kxmax,1));
 	for (int mz=0; mz<Mz; ++mz) {
-	  Complex d_dz(0.0, cz*u.kz(mz));
+	  int kz = u.kz(mz);
+	  Complex d_dz(0.0, 2*pi*kz/Lz*zero_last_mode(kz,kzmax,1));
 	  f.cmplx(mx,my,mz,i) 
 	    += 0.5*(d_dx*uu.cmplx(mx,my,mz,i0)+d_dz*uu.cmplx(mx,my,mz,i2));
 	}
@@ -1267,13 +1273,14 @@ void linearizedNL(const FlowField& u_, const ChebyCoeff& U_, FlowField& f) {
   int Mx = u.Mx();
   int Ny = u.Ny();
   int Mz = u.Mz();
-
-  Real ax = 2*pi/u.Lx();
+  Real Lx = u.Lx();
+  int kxmax = u.kxmax();
 
   u.makeState(Spectral, Physical);
   for (int ny=0; ny<Ny; ++ny) 
     for (int mx=0; mx<Mx; ++mx) {
-      Complex d_dx(0.0, ax*u.kx(mx));
+      int kx = u.kx(mx);
+      Complex d_dx(0.0, 2*pi*kx/Lx*zero_last_mode(kx,kxmax,1));
       for (int mz=0; mz<Mz; ++mz) {
 	cu = u.cmplx(mx,ny,mz, 0);
 	cv = u.cmplx(mx,ny,mz, 1);
@@ -1291,7 +1298,6 @@ void linearizedNL(const FlowField& u_, const ChebyCoeff& U_, FlowField& f) {
   return;
 }
 
-
 void uq2p(const FlowField& u_, const FlowField& q, FlowField& p, 
 	  NonlinearMethod nonl_method) {
   
