@@ -1535,6 +1535,8 @@ PPEDNS::PPEDNS(FlowField& u, const ChebyCoeff& Ubase,
   if (order_ > 0) { // should always be true
     u_[0] = u;
     navierstokesNL(u_[0], Ubase_, f_[0], tmp_, flags_.nonlinearity);
+    f_[0] *= -1.0;
+
     psolve_.solve(p, u);
     grad(p, tmp);
     f_[0] -= tmp;
@@ -1682,7 +1684,7 @@ void PPEDNS::advance(FlowField& un, FlowField& qn, int Nsteps) {
     }
 
     // The solution is stored in un. Shift entire u and f arrays in time
-    // and push un into u_[0]. I.e. shift u_[K] <- u_[K-1] <- ... <- u_[0] <- un
+    // and push un into u_[0]. Ie shift u_[K] <- u_[K-1] <- ... <- u_[0] <- un
     for (int j=order_-1; j>0; --j) {
       swap(f_[j], f_[j-1]);
       swap(u_[j], u_[j-1]);
@@ -1690,6 +1692,8 @@ void PPEDNS::advance(FlowField& un, FlowField& qn, int Nsteps) {
     if (order_ > 0) {
       u_[0] = un;
       navierstokesNL(u_[0], Ubase_, f_[0], tmp_, flags_.nonlinearity);
+      f_[0] *= -1.0;
+
       psolve_.solve(qn, u_[0]);
       grad(qn, tmp_);
       f_[0] -= tmp_;
