@@ -249,7 +249,7 @@ FlowField::FlowField(const string& filebase)
   }
 }
 
-FlowField::FlowField(const string& filebase, const string& chflow_version)
+FlowField::FlowField(const string& filebase, int major, int minor, int update)
   :
   Nx_(0),
   Ny_(0),
@@ -281,27 +281,47 @@ FlowField::FlowField(const string& filebase, const string& chflow_version)
     is.open(filebase.c_str(), ios::in | ios::binary);
 
   if (!is.good()) {
-    cerr << "FlowField::FlowField(filebase, version) : can't open file " 
+    cerr << "FlowField::FlowField(filebase, major,minor,update) : can't open file " 
 	 << filename << " or " << filebase << endl;
     exit(1);
   }
 
-  if (chflow_version == "0.9.16") {
+  if (major > 0 || minor > 9) {
+    cerr << "FlowField::FlowField(filebase, major,minor,update) :\n"
+	 << major << '.' << minor << '.' << update << " is higher than 0.9.x" << endl;
+    exit(1);
+  }
+  
+  if (update >= 16) {
     (*this) = FlowField(filebase); // not efficient but saves code redundancy
     return;
   }
-  else if (chflow_version != "0.9.15") {
-    readLittleEndian(is, Nx_);
-    readLittleEndian(is, Ny_);
-    readLittleEndian(is, Nz_);
-    readLittleEndian(is, Nd_);
-    read(is, xzstate_);
-    read(is, ystate_);
-    readLittleEndian(is, Lx_);
-    readLittleEndian(is, Lz_);
-    readLittleEndian(is, a_);
-    readLittleEndian(is, b_);
-    read(is, dealiasIO_);
+  else {
+    is.read((char*) &Nx_, sizeof(int));
+    is.read((char*) &Ny_, sizeof(int));
+    is.read((char*) &Nz_, sizeof(int));
+    is.read((char*) &Nd_, sizeof(int));
+    is >> xzstate_;
+    is >> ystate_;
+    is.read((char*) &Lx_, sizeof(Real));
+    is.read((char*) &Lz_, sizeof(Real));
+    is.read((char*) &a_, sizeof(Real));
+    is.read((char*) &b_, sizeof(Real));
+    char s;
+    is.get(s);
+    dealiasIO_ = (s=='1') ? true : false;
+
+    //readLittleEndian(is, Nx_);
+    //readLittleEndian(is, Ny_);
+    //readLittleEndian(is, Nz_);
+    //readLittleEndian(is, Nd_);
+    //read(is, xzstate_);
+    //read(is, ystate_);
+    //readLittleEndian(is, Lx_);
+    //readLittleEndian(is, Lz_);
+    //readLittleEndian(is, a_);
+    //readLittleEndian(is, b_);
+    //read(is, dealiasIO_);
     resize(Nx_, Ny_, Nz_, Nd_, Lx_, Lz_, a_, b_);
 
     // Read data only for non-aliased modes, assume 0 for aliased.
@@ -316,7 +336,8 @@ FlowField::FlowField(const string& filebase, const string& chflow_version)
 
 	  for (int nx=0; nx<=Nxd; ++nx) {
 	    for (int nz=0; nz<=Nzd; ++nz)
-	      readLittleEndian(is, rdata_[flatten(nx,ny,nz,i)]);
+	      is.read((char*) (rdata_ + flatten(nx,ny,nz,i)), sizeof(Real));
+	    //readLittleEndian(is, rdata_[flatten(nx,ny,nz,i)]);
 	    for (int nz=Nzd+1; nz<Nzpad_; ++nz)
 	      rdata_[flatten(nx,ny,nz,i)] = 0.0;
 	  }
@@ -326,7 +347,8 @@ FlowField::FlowField(const string& filebase, const string& chflow_version)
 	
 	  for (int nx=Nx_-Nxd; nx<Nx_; ++nx) {
 	    for (int nz=0; nz<=Nzd; ++nz)
-	      readLittleEndian(is, rdata_[flatten(nx,ny,nz,i)]);
+	      is.read((char*) (rdata_ + flatten(nx,ny,nz,i)), sizeof(Real));
+	    //readLittleEndian(is, rdata_[flatten(nx,ny,nz,i)]);
 	    for (int nz=Nzd+1; nz<Nzpad_; ++nz)
 	      rdata_[flatten(nx,ny,nz,i)] = 0.0;
 	  }
@@ -336,15 +358,10 @@ FlowField::FlowField(const string& filebase, const string& chflow_version)
     else {
       int N = Nd_*Ny_*Nx_*Nzpad_;
       for (int i=0; i<N; ++i)
-	readLittleEndian(is, rdata_[i]);
+	is.read((char*) (rdata_ + i), sizeof(Real));
+      //readLittleEndian(is, rdata_[i]);
     }
   }
-  else {
-    cerr << "FlowField::FlowField(filename, ch_version) : unknown version "
-	 << chflow_version << ".\n"
-	 << "Versioned binary IO is implemented for 0.9.15 and 0.9.16.\n";
-    exit(1);
-  }
 }
 
 Vector FlowField::xgridpts() const {
@@ -506,8 +523,8 @@ void FlowField::interpolate(const FlowField& ff) {
 
   fieldstate fxstate = f.xzstate();
   fieldstate fystate = f.ystate();
-  fieldstate gxstate = f.xzstate();
-  fieldstate gystate = f.ystate();
+  fieldstate gxstate = g.xzstate();
+  fieldstate gystate = g.ystate();
   
   // Always check these, debugging or not. 
   if (f.Lx() != g.Lx()  || f.Lz() != g.Lz() || f.a() != g.a()  ||
@@ -516,7 +533,7 @@ void FlowField::interpolate(const FlowField& ff) {
 	 << "FlowField doesn't match argument f geometrically.\n";
     exit(1);
   }
-  if (f.Nx()>g.Nx()  || f.Nz()>g.Nz()  || f.Nz()>g.Nz()) {
+  if (f.Nx()>g.Nx()  || f.Ny()>g.Ny()  || f.Nz()>g.Nz()) {
     cerr << "FlowField::interpolate(const FlowField& f) error:\n"
 	 << "interpolation requires Nx>=f.Nx, Ny>=f.Ny, Nz>=f.Nz\n";
     exit(1);
@@ -782,6 +799,23 @@ void FlowField::addProfile(const BasisFunc& profile, bool addconj) {
   }
 }
 
+void FlowField::translate(Real delta_x, Real delta_z) {
+  fieldstate xzs = xzstate_;
+  makeState(Spectral, ystate_);
+  for (int i=0; i<Nd_; ++i)
+    for (int ny=0; ny<Ny_; ++ny) 
+      for (int mx=0; mx<Mx(); ++mx) {
+	Complex cx = exp(Complex(0.0, 2*pi*kx(mx)*delta_x/Lx_));
+	for (int mz=0; mz<Mz(); ++mz) {
+	  Complex cz = exp(Complex(0.0, 2*pi*kz(mz)*delta_z/Lz_));
+	  cmplx(mx,ny,mz,i) *= cx*cz;
+	}
+      }
+  makeState(xzs, ystate_);
+}
+	  
+
+
 bool FlowField::geomCongruent(const FlowField& v) const {
   return (Nx_ == v.Nx_ &&
 	  Ny_ == v.Ny_ &&
@@ -979,7 +1013,7 @@ void FlowField::makeState(fieldstate xzstate, fieldstate ystate)  {
   (ystate  == Physical) ? makePhysical_y()  : makeSpectral_y();
 }
 
-Complex FlowField::Dx(int mx, int n) {
+Complex FlowField::Dx(int mx, int n) const {
   Complex rot(0.0, 0.0);
   switch(n % 4) {
   case 0: rot = Complex( 1.0, 0.0); break;
@@ -991,7 +1025,7 @@ Complex FlowField::Dx(int mx, int n) {
   int kx_ = kx(mx);
   return rot*(pow(2*pi*kx_/Lx_, n)*zero_last_mode(kx_,kxmax(),n));
 }
-Complex FlowField::Dz(int mz, int n) {
+Complex FlowField::Dz(int mz, int n) const {
   Complex rot(0.0, 0.0);
   switch(n % 4) {
   case 0: rot = Complex( 1.0, 0.0); break;
@@ -1004,11 +1038,11 @@ Complex FlowField::Dz(int mz, int n) {
   return rot*(pow(2*pi*kz_/Lz_, n)*zero_last_mode(kz_,kzmax(),n));
 }
 
-Complex FlowField::Dx(int mx) {
+Complex FlowField::Dx(int mx) const {
   int kx_ = kx(mx);
   return Complex(0.0, 2*pi*kx_/Lx_*zero_last_mode(kx_,kxmax(),1));
 }
-Complex FlowField::Dz(int mz) {
+Complex FlowField::Dz(int mz) const {
   int kz_ = kz(mz);
   return Complex(0.0, 2*pi*kz_/Lz_*zero_last_mode(kz_,kzmax(),1));
 }
